function SpaceScript(that){
	// Game Rules
	this.scene.setRule("gravity", 0);
	this.scene.setRule("isSnowing", false);
	this.scene.setRule("world", "space");
	this.scene.setRule("canShoot", true);
	this.scene.setRule("useBounds", true);
	this.scene.setRule("wave", 0);
	this.scene.setRule("score", 0);
	
	// Script Rules
	this.setQuality(0.5);
	
	var healthDisplay = "half";
	
	
	// Create Player
	var player = new Wizard(this);
	this.scene.setPlayer(player);
	this.scene.addObject(player);
	
	var waveIntro = null;
	
	this.stateUpdate = function(spawnOnly){
		// Check to see if the wave has been finished
		if(waveIntro) return;
		for(var i=0; i<this.scene.objects.length; i++) if(this.scene.objects[i].constructor===Alien) return;
		if(player.life < player.maxLife) player.life++;
		
		
		var ts1 = textSize(this, "Wave " + (that.scene.getRule("wave") + 1) + " IN", that.atlas.gui.font.white, 1),
			ts2 = textSize(this, "N", that.atlas.gui.font.white, 1.7);
		
		waveIntro = this.animations.add(function(i){
			if(i === 1){
				waveIntro = null;
				that.scene.setRule("wave", that.scene.rules.wave + 1);
				
				
				var newEnemyEveryNWaves = 2,
					
					spawnCount1 = 3,
					spawnCount10 = 30,
					
					selectMult1 = 0,
					selectMult20 = 20 / newEnemyEveryNWaves,
					
					budgetMult0 = 1,
					budgetMult5 = 15,
					
					spawnCountDelta = (spawnCount10 - spawnCount1) / 10,
					selectMultDelta = (selectMult20 - selectMult1) / 20;
					budgetMultDelta = (budgetMult5 - budgetMult0) / 5;
					
					wave = that.scene.getRule("wave") - 1,
					budget = Math.round(spawnCount1 + spawnCountDelta * wave),
					aliens = [];
				
				// Boss Wave
				if(wave % 15 === 0 && wave > 0){
					var boss = new Alien(that, 5);
					boss.x = 330/2;
					boss.y = 330/2;
					
					// Eject Player
					var distance = Math.hypot(330/2 - player.x, 330/2 - player.y),
					desiredR = Math.atan2(boss.y - player.y, boss.x - player.x) * (180 / Math.PI) - 90;
					
					player.vx += (boss.x - distance) / 30 *  Math.sin(desiredR/180*Math.PI),
					player.vy += (boss.y - distance) / 30 * -Math.cos(desiredR/180*Math.PI);
					
					boss.r = desiredR;
					that.scene.addObject(boss);
					
					// Spawn Explosions
					for(var j=0; j<15; j++){
						var offsetX = boss.scale * 4 + (Math.random() * 2 - 1) * 15 * 2,
							offsetY = boss.scale * 5 + (Math.random() * 2 - 1) * 30 * 2,
							
							x = boss.x,
							y = boss.y;
						
						x += offsetX *  Math.sin((boss.r - 90)/180*Math.PI);
						y += offsetX * -Math.cos((boss.r - 90)/180*Math.PI);
						
						x += offsetY *  Math.sin(boss.r/180*Math.PI);
						y += offsetY * -Math.cos(boss.r/180*Math.PI);
						
						var particle = new Explosion(that, that.textures.projectiles, that.atlas.particles.explosion.green, x, y, boss.r, boss.vx, boss.vy, boss.vr);
						
						particle.maxLife = Math.random() + 0.5;
						particle.life = particle.maxLife;
						that.scene.addParticle(particle)
					}
				}
				
				// Standard Wave
				else {
					for(var i=0; i<budget;){
						var type;
						
						do { type = Math.min(4, Math.floor(Math.random() * (selectMult1 + selectMultDelta * wave))); }
						while(i + type * budgetMultDelta + 1 > budget);
						
						if(type===1) i += 1 + 1;
						else if(type===5) didSpawnBoss = true;
						else i += type + 1;
						
						var alien = new Alien(that, type);
						that.scene.addObject(alien);
						aliens.push(alien);
					}
					
					var offsetR = Math.random() * 360,
					wideAngle = Math.random() < 0.25 ?  180 : 180  + 180 * Math.random();
					
					for(var i=0; i<aliens.length; i++){
						var alien = aliens[i],
							angle = i/aliens.length * wideAngle + offsetR,
							step = 330/2 * 1.5;
						
						alien.x = 330/2 + step *  Math.sin(angle/180*Math.PI),
						alien.y = 330/2 + step * -Math.cos(angle/180*Math.PI);
						alien.r = Math.random() * 360;
					}
				}
			}
		}, function(i){
			var m = i < 0.5 ? i : 1 - i,
				oy = -i * 10 + 10;
			
			that.context.globalAlpha = m * 20;
			drawText(that.context, that.textures, that.atlas, that.atlas.gui.font.white, "WAVE " + (that.scene.getRule("wave") + 1) + " IN", that.ratio.width/that.scale/2 - ts1.width/2, that.ratio.height/that.scale/2 + 30 + oy, 1);
			drawText(that.context, that.textures, that.atlas, that.atlas.gui.font.white, Math.ceil(5 * (1 - i)).toString(), that.ratio.width/that.scale/2 - ts2.width/2, that.ratio.height/that.scale/2 + ts1.height + 35 + oy, 1.7);
			that.context.globalAlpha = 1;
		}, 5);
	};
	
	
	// Zoom inward
	var initialZoom = 0.5,
		finalZoom = 2;
	
	this.animations.add(function(i){ this.scale = initialZoom + (finalZoom - initialZoom) * ease(i); }, null, 30);
	
	
	this.stateDraw = function(context, textures, atlas){
		var map, offsetX, offsetY, scale, subScale, b,
			wave = this.scene.getRule("wave");
		
		var tt3 = "SCORE: " + that.scene.getRule("score"),
			ts3 = textSize(this, tt3, that.atlas.gui.font.white, 1/this.scale);
		
		context.globalAlpha = 0.5;
		drawText(context, textures, atlas, atlas.gui.font.white, tt3, Math.floor(this.ratio.width/this.scale - ts3.width - 4), 12, 1/this.scale);
		context.globalAlpha = 1;
		
		// Display the health monitor
		var spacing = 2;
		scale = 1/1.4;
		offsetX = this.ratio.width/this.scale/ 2,
		offsetY = 2;
		
		if(healthDisplay==="half"){
			offsetX -= player.maxLife * atlas.gui.hp[0][3] * scale / 4 + ((player.maxLife/4) - 1) * spacing / 2;
			
			for(var i=0; i<player.maxLife/2; i++){
				a = player.shield / player.shieldMax;
				map = i >= player.life/2 ? atlas.gui.hp[2] : (i + 0.5  === player.life/2 ? atlas.gui.hp[1] : atlas.gui.hp[0]);
				context.drawImage(textures.gui, map[0], map[1], map[2], map[3], i * (map[2] * scale + spacing) + offsetX, offsetY, map[2] * scale, map[3] * scale);
				if(a > 0) context.drawImage(textures.gui, map[0], map[1] + map[3] * (1 - a) + 13, map[2], map[3] * a, i * (map[2] * scale + spacing) + offsetX, offsetY + map[3] * scale * (1 - a), map[2] * scale, map[3] * scale * a);
			}
		}
		
		else if(healthDisplay==="full"){
			offsetX -= player.maxLife * atlas.gui.hp[0][3] / 2 + (player.maxLife - 1) * spacing / 2;
			
			for(var i=0; i<player.maxLife; i++){
				map = i >= player.life ? atlas.gui.hp[2] : atlas.gui.hp[0];
				context.drawImage(textures.gui, map[0], map[1], map[2], map[3], i * (map[2] + spacing) + offsetX, offsetY, map[2], map[3]);
			}
		}
		
		
		var thisSpell = player.spellIndex,
			prevSpell = mod(player.spellIndex - 1, player.spells.length),
			nextSpell = mod(player.spellIndex + 1, player.spells.length);
		
		
		scale = 1;
		subScale = 1/1.4;
		offsetX = 6;
		offsetY = 6;
		
		
		// Display the previous spell selection
		map = atlas.gui.spells[prevSpell];
		context.drawImage(textures.gui, map[0], map[1], map[2], map[3], offsetX - map[2] * subScale + 4 * scale, offsetY + map[3] * scale - 2 * scale, map[2] * subScale, map[3] * subScale);
		
		context.globalAlpha = 0.45;
		map = atlas.gui.spellShadow;
		context.drawImage(textures.gui, map[0], map[1], map[2], map[3], offsetX - map[2] * subScale + 4 * scale, offsetY + map[3] * scale - 2 * scale, map[2] * subScale, map[3] * subScale);
		context.globalAlpha = 1;
		
		
		// Display the current spell selection
		map = atlas.gui.spells[thisSpell];
		context.drawImage(textures.gui, map[0], map[1], map[2], map[3], offsetX, offsetY, map[2] * scale, map[3] * scale);
		
		
		// Display the cooldown timer
		if(a = player.spellCooldown / player.spellCooldownMax){
			map = atlas.gui.spellCooldown;
			context.drawImage(textures.gui, map[0], map[1] + (1 - a) * map[3], map[2], a * map[3], offsetX, offsetY + (1 - a) * map[3], map[2], a * map[3]);
		}
		
		
		// Display the next spell selection
		map = atlas.gui.spells[nextSpell];
		context.drawImage(textures.gui, map[0], map[1], map[2], map[3], offsetX + map[2] * scale - 2 * scale, offsetY - map[3] * subScale + 4 * scale, map[2] * subScale, map[3] * subScale);
		
		context.globalAlpha = 0.45;
		map = atlas.gui.spellShadow;
		context.drawImage(textures.gui, map[0], map[1], map[2], map[3], offsetX + map[2] * scale - 2 * scale, offsetY - map[3] * subScale + 4 * scale, map[2] * subScale, map[3] * subScale);
		context.globalAlpha = 1;
		
		
		// Display the wave counter
		var cw = atlas.gui.numbers[0][2],
			ch = atlas.gui.numbers[0][3],
			waveStr = wave.toString();
		
		scale = 1/1.4;
		map = atlas.gui.wave;
		offsetX = this.ratio.width/this.scale - map[2] * scale - spacing - (waveStr.length + 1) * cw;
		offsetY = spacing + 2;
		
		context.drawImage(textures.gui, map[0], map[1], map[2], map[3], offsetX + 2, offsetY, map[2] * scale, map[3] * scale);
		offsetX += map[2] * scale;
		
		for(var i=0; i<waveStr.length; i++){
			map = atlas.gui.numbers[waveStr[i]];
			context.drawImage(textures.gui, map[0], map[1], map[2], map[3], offsetX + (i + 1) * cw, offsetY, map[2] * scale, map[3] * scale);
		}
	};
}
